using System;
using System.Collections.Generic;
using System.Text;
using System.Query;
using System.Xml.XLinq;

namespace CSDemo
{
    public static class CustomExtenders
    {
        public static string ChangeCase(this string source)
        {
            return new System.Globalization.CultureInfo("en-US").TextInfo.ToTitleCase(source);
        }

        public static XElement ToXml<T>(this IEnumerable<T> source, XName name)
        {
            return new XElement(name,
                from item in source select new XElement("item", item));
        }

/* Why Extension Methods:
 * Extension Methods are useful because without them we would have to 
 * write code which is more difficult to comprehend.
 * With Extension methods we can write:
 * 
 *      var results = Customers
 *          .Where(c => c.City == "London")
 *          .OrderBy(c => c.CompanyName)
 *          .Select(c => c.CompanyName)
 *          .ToArray
 *          
 * Without extension methods, we would have to write the following:
         var results3 = System.Query.Sequence.ToArray(
                                  System.Query.Sequence.Select(
                                    System.Query.Sequence.OrderBy(
                                        System.Query.Sequence.Where(db.Customers
                                        , c => c.City == "London")
                                    , c => c.CompanyName)
                                  , c => c.CompanyName)
                                );
        */
    }

     public static class CustomExtenders2
    {
        public static string ChangeCase(this object source)
        {
            return source.ToString().ToUpper();
        }
     }
}
