''' <summary>
''' Make an rss feed out of the event logs using LINQ to XML and VB9
''' </summary>
''' <remarks>External plumbing code taken from 
''' at www.codeproject.com/aspnet/EventLogRss.asp
''' Custom WriteRSS method is original code</remarks>

Partial Class _Default
    Inherits System.Web.UI.Page

    Private LogName As String = "Application"
    Private EventSourceName As String = ""
    Private FilterType As String = ""

    ''' <summary>
    ''' Page Load
    ''' </summary>
    ''' <remarks>Set up the encoding type to return XML rather than HTML
    ''' We also check the querystring for source and type parameters for the filtering.</remarks>
    Protected Sub Page_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        EventSourceName = Request.QueryString("source")
        FilterType = Request.QueryString("type")
        Response.Buffer = True
        Response.Clear()
        Response.Cache.SetCacheability(HttpCacheability.NoCache)
        Response.ContentType = "text/xml"
        Response.AddHeader("Content-Disposition", "inline;filename=" + LogName + "_log.xml")
        WriteRss()
        Response.End()
    End Sub

    ''' <summary>
    ''' Write the RSS representation of the event log to the response stream
    ''' </summary>
    ''' <remarks>Uses LINQ and VB9's XML literals to declaratively create the
    ''' RSS XML. Technically, this represents only 1 line of code. The rest is continuation chars.
    ''' Since log.Entries does not implement IEnumerable(Of EventLogEntry), force it using .Cast</remarks>
    Private Sub WriteRss()
        If EventLog.Exists(LogName) Then _
            Response.Write(<channel>
                               <title>Event Log from <%= Environment.MachineName %></title>
                               <description>Events logged in the system's <%= LogName %> Event Log</description>
                               <copyright>This RSS feed is copyright (c) <%= Now.Year.ToString %>.</copyright>
                               <publisher>Your Name</publisher>
                               <author>you Lastname</author>
                               <language>en-US</language>
                               <%= From entry In New EventLog(LogName).Entries.Cast(Of EventLogEntry)() _
                                   Where (FilterType Is Nothing OrElse FilterType.ToUpper = entry.EntryType.ToString.ToUpper) And _
                                   (EventSourceName Is Nothing OrElse EventSourceName.ToUpper = entry.Source.ToUpper) _
                                   Select <item>
                                           <title><%= entry.EntryType.ToString() & ": " & entry.Source %></title>
                                           <guid isPermaLink="false"><%= Environment.MachineName & "/" & LogName & "/" & entry.Index %></guid>
                                           <description><%= GetItemText(entry) %></description>
                                           <pubDate><%= entry.TimeWritten.ToUniversalTime().ToString() %></pubDate>
                                           <date><%= entry.TimeWritten.ToString() %></date>
                                           <creator><%= entry.UserName %></creator>
                                       </item> %>
                           </channel>)
    End Sub

    ''' <summary>
    ''' Predefined template for the description
    ''' </summary>
    ''' <remarks>We will use String.Format to fill the values into the placeholders. 
    ''' when we take out the continuation chars, this is only 1 line of code </remarks>
    Private Const itemFormat As String = "<h4>The following event has been logged in the {0} Event Log of {1}</h4>" & _
        "<b>Event time:</b> {2:yyyy-MM-dd hh:mm:ss tt zzz} <br>" & _
        "<b>Event type:</b> {3} <br>" & _
        "<b>Event source:</b> {4} <br>" & _
        "<b>User:</b> {5} <br>" & _
        "<b>Event ID:</b> {6} <br>" & _
        "<pre>{7}</pre>"

    ''' <summary>
    ''' Get text for description element
    ''' </summary>
    ''' <param name="entry">A single event log entry</param>
    ''' <returns>String formatted with html formatting for the description of the element, including the encoded entry message details</returns>
    ''' <remarks></remarks>
    Private Function GetItemText(ByVal entry As EventLogEntry) As String
        Return String.Format(itemFormat, LogName, Environment.MachineName, entry.TimeWritten, entry.EntryType, _
            entry.Source, entry.UserName, entry.InstanceId, System.Web.HttpUtility.HtmlEncode(entry.Message))
    End Function
End Class