Imports Microsoft.VisualBasic
Imports CookComputing.XmlRpc

Namespace MetaWeblog
    <XmlRpcMissingMapping(MappingAction.Ignore)> _
    Public Structure Blog
        Public Uri As String
        Public Title As String
    End Structure

    <XmlRpcMissingMapping(MappingAction.Ignore)> _
    Public Structure Enclosure
        Public length As Integer
        Public type As String
        Public url As String
    End Structure

    <XmlRpcMissingMapping(MappingAction.Ignore)> _
    Public Structure Source
        Public name As String
        Public url As String
    End Structure

    <XmlRpcMissingMapping(MappingAction.Ignore)> _
    Public Structure Post
        <XmlRpcMissingMapping(MappingAction.Error), XmlRpcMember(description:="Required when posting.")> _
        Public dateCreated As DateTime
        <XmlRpcMissingMapping(MappingAction.Error), XmlRpcMember(description:="Required when posting.")> _
        Public description As String
        <XmlRpcMissingMapping(MappingAction.Error), XmlRpcMember(description:="Required when posting.")> _
        Public title As String
        Public categories() As String
        Public enclosure As Enclosure
        Public permalink As String
        <XmlRpcMember(description:="Not required when posting. Depending on server may " & _
            "be either string or integer. " & _
            "Use Convert.ToInt32(postid) to treat as integer or " & _
            "Convert.ToString(postid) to treat as string")> _
        Public postid As Object
        Public source As Source
        Public userid As String
        Public mt_allow_comments As Object
        Public mt_allow_ping As Object
        Public mt_convert_breaks As Object
        Public mt_text_more As String
        Public mt_excerpt As String
    End Structure

    Public Structure CategoryInfo
        Public description As String
        Public htmlUrl As String
        Public rssUrl As String
        Public title As String
        Public categoryid As String
    End Structure

    Public Structure Category
        Public categoryId As String
        Public categoryName As String
    End Structure

    Public Structure FileData
        Public bits() As Byte
        Public name As String
        Public type As String
    End Structure

    Public Structure UrlData
        Public url As String
    End Structure

    Public Interface IMetaWeblog
        <XmlRpcMethod("blogger.getUserBlogs", Description:="Retrievs a list of blogs for a given user. Returns array of blogs()")> _
        Function getUserBlogs() As Blog()

        <XmlRpcMethod("metaWeblog.editPost", Description:="Updates and existing post to a designated blog using the metaWeblog API. Returns true if completed.")> _
        Function editPost( _
            ByVal postid As String, _
            ByVal username As String, _
            ByVal password As String, _
            ByVal post As Post, _
            ByVal publish As Boolean) As Object

        <XmlRpcMethod("metaWeblog.getCategories", Description:="Retrieves a list of valid categories for a post using the metaWeblog API. Returns the metaWeblog categories struct collection.")> _
        Function getCategories( _
            ByVal blogid As String, _
            ByVal username As String, _
            ByVal password As String) As CategoryInfo()

        <XmlRpcMethod("metaWeblog.getPost", Description:="Regrieves an existing post using the metaWeblog API. Returns the metaWeblog struct.")> _
        Function getPost( _
            ByVal postid As String, _
            ByVal username As String, _
            ByVal password As String) As Post

        <XmlRpcMethod("metaWeblog.getRecentPosts", Description:="Retrieves a list of the most recent existing post using the metaWeblog API. Returns the metaWeblog struct collection")> _
        Function getRecentPosts( _
            ByVal blogid As String, _
            ByVal username As String, _
            ByVal password As String, _
            ByVal numberOfPosts As Integer) As Post()

        <XmlRpcMethod("metaWeblog.newPost", Description:="Makes a new post to a designated blog using the metaWeblog API. Returns postid as a string.")> _
        Function newPost( _
            ByVal blogid As String, _
            ByVal username As String, _
            ByVal password As String, _
            ByVal post As Post, _
            ByVal publish As Boolean) As String

        <XmlRpcMethod("metaWeblog.newMediaObject", Description:="Makes a new file to a designated blog using the metaWeblog API. Returns url as a string of a struct.")> _
        Function newMediaObject( _
            ByVal blogid As String, _
            ByVal username As String, _
            ByVal password As String, _
            ByVal file As FileData) As UrlData

    End Interface
End Namespace
