Public Class Customer

    Private _CustomerID As String
    Private _CompanyName As String
    Private _ContactName As String
    Private _ContactTitle As String
    Private _Address As String
    Private _City As String
    Private _Region As String = ""
    Private _PostalCode As String = ""
    Private _Country As String
    Private _Phone As String
    Private _Fax As String = ""
    Private _OrderList As New List(Of OrderItem)

    Public Property CustomerID() As String
        Get
            Return _CustomerID
        End Get
        Set(ByVal value As String)
            _CustomerID = value
        End Set
    End Property

    Public Property CompanyName() As String
        Get
            Return _CompanyName
        End Get
        Set(ByVal value As String)
            _CompanyName = value
        End Set
    End Property

    Public Property ContactTitle() As String
        Get
            Return _ContactTitle
        End Get
        Set(ByVal value As String)
            _ContactTitle = value
        End Set
    End Property

    Public Property Address() As String
        Get
            Return _Address
        End Get
        Set(ByVal value As String)
            _Address = value
        End Set
    End Property

    Public Property City() As String
        Get
            Return _City
        End Get
        Set(ByVal value As String)
            _City = value
        End Set
    End Property

    Public Property Region() As String
        Get
            Return _Region
        End Get
        Set(ByVal value As String)
            _Region = value
        End Set
    End Property

    Public Property PostalCode() As String
        Get
            Return _PostalCode
        End Get
        Set(ByVal value As String)
            _PostalCode = value
        End Set
    End Property

    Public Property Country() As String
        Get
            Return _Country
        End Get
        Set(ByVal value As String)
            _Country = value
        End Set
    End Property

    Public ReadOnly Property Orders() As List(Of OrderItem)
        Get
            Return _OrderList
        End Get
    End Property

    Public Property Phone() As String
        Get
            Return _Phone
        End Get
        Set(ByVal value As String)
            _Phone = value
        End Set
    End Property

    Public Property Fax() As String
        Get
            Return _Fax
        End Get
        Set(ByVal value As String)
            _Fax = value
        End Set
    End Property

    Public Sub New()
        'force instantiation via factory
    End Sub

    Public Shared Function GetCustomer(ByVal dr As IDataReader) As Customer
        Dim child As New Customer
        child.Fetch(dr)
        Return child
    End Function

    Friend Sub Fetch(ByVal dr As IDataReader)
        With dr
            _CustomerID = dr.GetString(0)
            _CompanyName = dr.GetString(1)
            _ContactName = dr.GetString(2)
            _ContactTitle = dr.GetString(3)
            _Address = dr.GetString(4)
            _City = dr.GetString(5)
            If Not dr.IsDBNull(6) Then _Region = dr.GetString(6)
            If Not dr.IsDBNull(7) Then _PostalCode = dr.GetString(7)
            _Country = dr.GetString(8)
            _Phone = dr.GetString(9)
            If Not dr.IsDBNull(10) Then _Fax = dr.GetString(10)
        End With
    End Sub
End Class

Public Class CustomerList
    Inherits List(Of Customer)

    Private Sub New()

    End Sub

    Public Function ItemByCustomerId(ByVal Id As String) As Customer
        For Each child As Customer In Me
            If child.CustomerID = Id Then
                Return child
            End If
        Next
        'Not Found
        Return Nothing
    End Function

    Public Shared Function GetCustomersAndOrders() As List(Of Customer)
        Dim list As New CustomerList
        List.FetchData(True)
        Return List
    End Function

    Public Shared Function GetCustomers() As List(Of Customer)
        Dim list As New CustomerList
        list.FetchData(False)
        Return list
    End Function

    Private Sub FetchData(ByVal loadChildren As Boolean)
        Using cn As New Data.SqlClient.SqlConnection("Data Source=.\SQLEXPRESS;AttachDbFilename=C:\Projects\Data\NORTHWND.MDF;Integrated Security=True;Connect Timeout=30;User Instance=True")
            cn.Open()
            Using cmCustomer As New Data.SqlClient.SqlCommand("Select * from customers", cn)
                Using dr As Data.SqlClient.SqlDataReader = cmCustomer.ExecuteReader
                    While dr.Read
                        Me.Add(Customer.GetCustomer(dr))
                    End While
                End Using 'dr
            End Using 'cmCustomer
            If loadChildren Then
                Using cmOrder As New Data.SqlClient.SqlCommand("Select * from Orders", cn)
                    Using dr As Data.SqlClient.SqlDataReader = cmOrder.ExecuteReader
                        While dr.Read
                            'find the customer for this order and add to their order item list
                            Dim cust As Customer = Me.ItemByCustomerId(dr.GetString(dr.GetOrdinal("CustomerId")))
                            cust.Orders.Add(OrderItem.GetOrder(dr))
                        End While
                    End Using
                End Using
            End If
            cn.Close()
        End Using 'cn
    End Sub
End Class
