Option Explicit On
Option Strict On

Imports System.IO
Imports System.Query
Imports DemoData
Imports DemoData.Dumper


Module Module1

    Sub Main()
        '1: Starting with standard iterators
        'For Each f As FileInfo In New DirectoryInfo("C:\windows").GetFiles
        '    Console.WriteLine(f.Name)
        'Next

        '1.1: Filtering the old way
        'For Each f As FileInfo In New DirectoryInfo("C:\windows").GetFiles
        '    If f.Extension = ".exe" Then
        '        Console.WriteLine(f.Name)
        '    End If
        'Next

        '2: query expressions
        'Dim results As IEnumerable(Of FileInfo) = _
        '     From f As FileInfo In New DirectoryInfo("C:\windows").GetFiles _
        '    Where f.Extension = ".exe" _
        '    Select f _
        '    Order By f.LastAccessTime Descending

        '2.1: Selecting isolated properties
        'Dim results As IEnumerable(Of FileStruct) = _
        '    From f As FileInfo In New DirectoryInfo("C:\windows").GetFiles _
        '    Where f.Extension = ".exe" _
        '    Select New FileStruct(f.Name, f.LastAccessTime) _
        '    Order By f.LastAccessTime Descending

        '2.2: Object initializers
        'Dim results As IEnumerable(Of FileStruct) = _
        '    From f As FileInfo In New DirectoryInfo("C:\windows").GetFiles _
        '    Where f.Extension = ".exe" _
        '    Select New FileStruct {FileName := f.Name, LastAccess := f.LastAccessTime} _
        '    Order By f.LastAccessTime Descending

        '2.3: Infered types
        'Dim results = _
        '    From f In New DirectoryInfo("C:\windows").GetFiles _
        '    Where f.Extension = ".exe" _
        '    Select New {f.Name, f.LastAccessTime, f.Extension} _
        '    Order By f.LastAccessTime Descending

        '2.4: Joining multiple sources 
        'Dim results = _
        '    From f In New DirectoryInfo("C:\windows").GetFiles, _
        '        e In FileTypes.Load _
        '    Where f.Extension = ".exe" And f.Extension = e.Extension _
        '    Select New {f.Name, f.LastAccessTime, e.Description} _
        '    Order By lastaccesstime Descending

        '3.0: reflection
        'Dim results = From e In [Enum].GetNames(GetType(ConsoleColor)) _
        '    Where e.StartsWith("Dark") _
        '    Select e _
        '    Order By e Descending

        'Dumper.Write(results)
        Console.ReadLine()
    End Sub

    Public Sub FileDemo()
        Dim result = _
            From f In New DirectoryInfo("C:\Windows").GetFiles(), _
                        e As FileType In FileTypes.Load _
            Where (f.Extension = ".exe" Or f.Extension = ".log") And f.Extension = e.Extension _
            Select New {f.Name, f.LastAccessTime, e.Description} _
            Order By LastAccessTime
        result.Write()

    End Sub
    Public Sub ReflectionDemo()
        Dim colors = From s In [Enum].GetNames(GetType(System.ConsoleColor)) _
            Where s.StartsWith("Dark") _
            Select s _
            Order By s Descending
        Dumper.Write(colors)
    End Sub
    Public Sub XLinqDemo()
        Dim val = <XLinqRocksBecause>
                      <LinqEssentials>
                          <Category>LINQ</Category>
                          <Category>DLINQ</Category>
                          <Category>XLINQ</Category>
                      </LinqEssentials>
                      <LanguageFeatures forDate=<%= Today() %>>
                          <release version="2.0">
                              <feature>Generics</feature>
                              <feature>Iterators</feature>
                              <feature>Anonymous Delegates</feature>
                              <feature>Nullable Types</feature>
                          </release>
                          <release version="3.0">
                              <feature>Implicit Types</feature>
                              <feature>Extension methods</feature>
                              <feature>Lambda Expressions</feature>
                              <feature>Object initializers</feature>
                              <feature>Anonymous Types</feature>
                              <feature>Expression Trees</feature>
                          </release>
                          <release version="9.0">
                              <feature>dynamic interfaces</feature>
                              <feature>dynamic identifiers</feature>
                              <feature>XML Literals</feature>
                          </release>
                      </LanguageFeatures>
                  </XLinqRocksBecause>
        Console.Write(val)

        Dim filter = val...<feature>
        filter.Write()
    End Sub
End Module


