Option Strict On

Imports System.IO

Module Module1

    Sub Main()

        'Standard Iterator
        'For Each f As String In Directory.GetFiles("C:\windows")
        '    Console.WriteLine(f)
        'Next

        'Filtered Iterator
        'For Each f As String In Directory.GetFiles("C:\windows")
        '    If Path.GetExtension(f) = ".exe" Then
        '        Console.WriteLine(f)
        '    End If
        'Next

        'Query syntax
        'Dim results As IEnumerable(Of String) = From f In Directory.GetFiles("C:\windows") _
        '    Where Path.GetExtension(f) = ".exe" _
        '    Select f _
        '    Order By File.GetLastAccessTime(f) Descending

        'Returning a custom type
        'Dim results As IEnumerable(Of CustomFile) = From f In Directory.GetFiles("C:\windows") _
        '    Where Path.GetExtension(f) = ".exe" _
        '    Select New CustomFile(f, File.GetLastAccessTime(f)) _
        '    Order By Accessdate Descending

        'Custom type doesn't implement an overloaded constructor, use Object Initializers
        'Dim results As IEnumerable(Of CustomFile) = From f In Directory.GetFiles("C:\windows") _
        '    Where Path.GetExtension(f) = ".exe" _
        '    Select New CustomFile {FileName := f, AccessDate := File.GetLastAccessTime(f)} _
        '    Order By Accessdate Descending

        'Anonymous Types
        'Dim results = From f In Directory.GetFiles("C:\windows") _
        '    Where Path.GetExtension(f) = ".exe" _
        '    Select New {FileName := f, AccessDate := File.GetLastAccessTime(f)} _
        '    Order By Accessdate Descending

        'another query sample: reflection
        'Dim results = From c In Type.GetType("System.ConsoleColor").GetFields _
        '    Where c.Name.StartsWith("Dark") _
        '    Select c.Name

        '        ObjectDumper.Write(results)
        Console.ReadKey()
    End Sub

    Private Sub XLinqDemo()
        Dim x = <LinqRocksBecause onDate=<%= Today() %>>
                    <LinqEssentials>
                        <Category>LINQ</Category>
                        <Category>DLINQ</Category>
                        <Category>XLINQ</Category>
                    </LinqEssentials>
                    <LanguageFeatures>
                        <release version="2.0">
                            <feature>Generics</feature>
                            <feature>Iterators</feature>
                            <feature>Anonymous Delegates</feature>
                            <feature>Nullable Types</feature>
                        </release>
                        <release version="3.0">
                            <feature>Implicit Types</feature>
                            <feature>Extension methods</feature>
                            <feature>Lambda Expressions</feature>
                            <feature>Object initializers</feature>
                            <feature>Anonymous Types</feature>
                            <feature>Expression Trees</feature>
                        </release>
                        <release version="9.0">
                            <feature>dynamic interfaces</feature>
                            <feature>dynamic identifiers</feature>
                            <feature>XML Literals</feature>
                        </release>
                    </LanguageFeatures>
                </LinqRocksBecause>
        Console.Write(x)
        For Each y In x.Descendants("feature")
            Console.WriteLine(y)
        Next
        Console.ReadKey()

    End Sub

    Private Sub CombinedDemo()
        'Dim db As nwind.Northwind = New nwind.Northwind("C:\Projects\Data\Northwnd.mdf")

        'Dim result = <Performance>
        '                 <%= Select <Cust><name><%= e.FirstName %><%= " " %><%= e.LastName %></name>
        '                         <orderCount><%= e.Orders.Count %></orderCount></Cust> _
        '                     From e In db.Employees _
        '                     Where e.Orders.Count > 100% %>
        '             </Performance>

        'Console.Write(result)
        'Console.ReadKey()
    End Sub

End Module